%----------------------------------------------------------------------------------------
% Recirculating 4-compartment model for O2-CO2 transport and exchange between the lungs 
% and a systemic organ, say heart. Instantaneous equilibrium binding of O2 and CO2 with 
% the hemoglobin (Hb) inside red blood cells (RBCs) is considered. The subscripts "Alv", 
% "PulCap", "SysCap", and "SysTis" indicate the four comaprtments "alveoli", "pulmonary 
% capillary", "systemic capillary", and "systemic tissue", respectively. The model solves
% the dynamics of four state variables (O2, CO2, HCO3M, HP) in the four compartments. The
% aleveoli compartment does not have HCO3M and HP, resulting in 14 mass conservation ODEs 
%
% Developed by: Ranjan K. Dash, Ph.D. (Last modified: 02/29/2016)
% Department of Physiology and Biotechnology and Bioengineering Center
% Medical College of Wisconsin, Milwaukee, WI-53226
%----------------------------------------------------------------------------------------

function [dydt,Output1,Output2] = Gas_Recirc(~,y)

% Specify anatomical, physiological, transport, exchange, and reaction parameters
load CommonParams.mat % Parameters that are common with the main routine

% Define all the state variables that are solved from the mass balance equations (ODEs)
FO2Alv = y(1); TO2PulCap = y(2); TO2SysCap = y(3); TO2SysTis = y(4);
FCO2Alv = y(5); TCO2PulCap = y(6); TCO2SysCap = y(7); FCO2SysTis = y(8);
HCO3mPulCap = y(9); HCO3mSysCap = y(10); HCO3mSysTis = y(11);
HpPulCap = y(12); HpSysCap = y(13); HpSysTis = y(14);

% Compute other state variables those define the mass balance equations (ODEs)
% Compute H+ (pH) and HCO3m in different compartments (PulCap(pl,rbc), SysCap(pl,rbc), 
% and SysTis(isf,pc)) based on volume fractions of RBCs in blood and PCs in tissue) 
HpPulCapPl = HpPulCap/((1-Hct)+Hct/GDrbc); HpPulCapRBC = HpPulCap/((1-Hct)*GDrbc+Hct);
HpSysCapPl = HpSysCap/((1-Hct)+Hct/GDrbc); HpSysCapRBC = HpSysCap/((1-Hct)*GDrbc+Hct);
HpSysTisISF = HpSysTis/((1-Hct2)+Hct2/GDpc); HpSysTisPC = HpSysTis/((1-Hct2)*GDpc+Hct2);

HCO3mPulCapPl = HCO3mPulCap/((1-Hct)+Hct*GDrbc); HCO3mPulCapRBC = HCO3mPulCap/((1-Hct)/GDrbc+Hct);
HCO3mSysCapPl = HCO3mSysCap/((1-Hct)+Hct*GDrbc); HCO3mSysCapRBC = HCO3mSysCap/((1-Hct)/GDrbc+Hct);
HCO3mSysTisISF = HCO3mSysTis/((1-Hct2)+Hct2*GDpc); HCO3mSysTisPC = HCO3mSysTis/((1-Hct2)/GDpc+Hct2);

pHPulCap = -log10(HpPulCap); pHSysCap = -log10(HpSysCap); pHSysTis = -log10(HpSysTis);
pHPulCapPl = -log10(HpPulCapPl); pHPulCapRBC = -log10(HpPulCapRBC); 
pHSysCapPl = -log10(HpSysCapPl); pHSysCapRBC = -log10(HpSysCapRBC);
pHSysTisISF = -log10(HpSysTisISF); pHSysTisPC = -log10(HpSysTisPC);

% Compute free O2 and CO2 (or PO2 and PCO2) from total O2 and CO2 (TO2 and TCO2) in various 
% compartments through numerical inversion method that uses SHbO2CO2_EJAP2016 routine 
PO2Alv = FO2Alv/alphaO2; PCO2Alv = FCO2Alv/alphaCO2;
[PCO2PulCap,FCO2PulCap] = TotCO2_FreeCO2_Inversion(TCO2PulCap,100,pHPulCapRBC,DPGrbc,Temp,HbRbc,Hct);
[PCO2SysCap,FCO2SysCap] = TotCO2_FreeCO2_Inversion(TCO2SysCap,40,pHSysCapRBC,DPGrbc,Temp,HbRbc,Hct);
[PO2PulCap,FO2PulCap] = TotO2_FreeO2_Inversion(TO2PulCap,PCO2PulCap,pHPulCapRBC,DPGrbc,Temp,HbRbc,Hct);
[PO2SysCap,FO2SysCap] = TotO2_FreeO2_Inversion(TO2SysCap,PCO2SysCap,pHSysCapRBC,DPGrbc,Temp,HbRbc,Hct);

% Calculate free O2 or PO2 from total O2 (free + myoglobin bound O2) in the systemic tissue 
FO2SysTis = (-(C50Mb+MbTis-TO2SysTis) + ((C50Mb+MbTis-TO2SysTis)^2 + 4*C50Mb*TO2SysTis)^0.5)/2;
PO2SysTis = FO2SysTis/alphaO2; PCO2SysTis = FCO2SysTis/alphaCO2;

% O2 consumption and CO2 production rates in the systemic tissue
KmO2Cox = 7e-7; % Km of O2 for cytocrome c oxydase in systemic tissue; M
VmaxCox = VdotO2/Mol2L; % Vmax of cytocrome c oxydase in systemic tissue; mol/min
MRO2SysTis = VmaxCox*FO2SysTis/(KmO2Cox+FO2SysTis); % O2 consumption rate in systemic tissue; mol/min
MRCO2SysTis = RQ*MRO2SysTis; % CO2 production rate in systemic tissue; mol/min

% Kinetic parameters for CO2 hydration reaction
K1 = 10^(-6.12);       % K (equilibrium constant) for CO2 hydration reaction; M
kp1 = 60*0.12;         % CO2 hydration reaction forward rate constant; 1/min
km1 = kp1/K1;          % CO2 hydration reaction reverse rate constant; 1/M/min

% ODEs for O2 (free or total), CO2 (free or total), bicarbonate ions, and hydrogen ions in
% the alveoli, pulmonary capillary, systemic capillary, and systemic tissue compartments.
dFO2Alv = ((VdotAlv/Mol2L)*(PIO2-PO2Alv)/(Pbaro-PH2O) - PSO2PulCap*(FO2Alv-FO2PulCap))/VolAlv;
dTO2PulCap = (Qdot*(TO2SysCap-TO2PulCap) + PSO2PulCap*(FO2Alv-FO2PulCap))/VolPulCap;
dTO2SysCap = (Qdot*(TO2PulCap-TO2SysCap) - PSO2SysCap*(FO2SysCap-FO2SysTis))/VolSysCap;
dTO2SysTis = (PSO2SysCap*(FO2SysCap-FO2SysTis) - MRO2SysTis)/VolSysTis;

dFCO2Alv = (-(VdotAlv/Mol2L)*PCO2Alv/(Pbaro-PH2O) + PSCO2PulCap*(FCO2PulCap-FCO2Alv))/VolAlv;
dTCO2PulCap = (Qdot*(TCO2SysCap-TCO2PulCap) - PSCO2PulCap*(FCO2PulCap-FCO2Alv) ...
    - VolPulCap*CAPulCap*(kp1*FCO2PulCap-km1*HCO3mPulCap*HpPulCap))/VolPulCap;
dTCO2SysCap = (Qdot*(TCO2PulCap-TCO2SysCap) + PSCO2SysCap*(FCO2SysTis-FCO2SysCap) ...
    - VolSysCap*CASysCap*(kp1*FCO2SysCap-km1*HCO3mSysCap*HpSysCap))/VolSysCap;
dFCO2SysTis = (MRCO2SysTis - PSCO2SysCap*(FCO2SysTis-FCO2SysCap) ...
    - VolSysTis*CASysTis*(kp1*FCO2SysTis-km1*HCO3mSysTis*HpSysTis))/VolSysTis;

dHCO3mPulCap = (Qdot*(HCO3mSysCap-HCO3mPulCap) ...
    + VolPulCap*CAPulCap*(kp1*FCO2PulCap-km1*HCO3mPulCap*HpPulCap))/VolPulCap;
dHCO3mSysCap = (Qdot*(HCO3mPulCap-HCO3mSysCap) - PSHCO3mSysCap*(GDcap*HCO3mSysCap-HCO3mSysTis) ...
    + VolSysCap*CASysCap*(kp1*FCO2SysCap-km1*HCO3mSysCap*HpSysCap))/VolSysCap;
dHCO3mSysTis = (PSHCO3mSysCap*(GDcap*HCO3mSysCap-HCO3mSysTis) ...
    + VolSysTis*CASysTis*(kp1*FCO2SysTis-km1*HCO3mSysTis*HpSysTis))/VolSysTis;

dHpPulCap = (Qdot*(HpSysCap-HpPulCap) + VolPulCap*CAPulCap*(2.303/BetaPulCap)*HpPulCap* ...
    (kp1*FCO2PulCap-km1*HCO3mPulCap*HpPulCap))/VolPulCap;
dHpSysCap = (Qdot*(HpPulCap-HpSysCap) - PSHpSysCap*(HpSysCap-GDcap*HpSysTis) + VolSysCap* ...
    CASysCap*(2.303/BetaSysCap)*HpSysCap*(kp1*FCO2SysCap-km1*HCO3mSysCap*HpSysCap))/VolSysCap;
dHpSysTis = (PSHpSysCap*(HpSysCap-GDcap*HpSysTis) + VolSysTis*CASysTis*(2.303/BetaSysTis)* ...
    HpSysTis*(kp1*FCO2SysTis-km1*HCO3mSysTis*HpSysTis))/VolSysTis;

dydt = [dFO2Alv; dTO2PulCap; dTO2SysCap; dTO2SysTis; dFCO2Alv; ...
        dTCO2PulCap; dTCO2SysCap; dFCO2SysTis; dHCO3mPulCap; ...
        dHCO3mSysCap; dHCO3mSysTis; dHpPulCap; dHpSysCap; dHpSysTis];

Output1 = [PO2Alv; PO2PulCap; PO2SysCap; PO2SysTis; PCO2Alv; ...
           PCO2PulCap; PCO2SysCap; PCO2SysTis]';

Output2 = [pHPulCap; pHPulCapPl; pHPulCapRBC; pHSysCap; pHSysCapPl; ...
           pHSysCapRBC; pHSysTis; pHSysTisISF; pHSysTisPC; HCO3mPulCap; ...
           HCO3mPulCapPl; HCO3mPulCapRBC; HCO3mSysCap; HCO3mSysCapPl; ...
           HCO3mSysCapRBC; HCO3mSysTis; HCO3mSysTisISF; HCO3mSysTisPC]';
