close all;
clear all; clc;
format long e

%% Specify anatomical, physiological, transport, exchange, and reaction parameters
Temp = 37;                  % Standard physiological temperature, degC
alphaO20 = 1.4574e-6;       % Solubility of O2 in water at 37 C, M/mmHg
alphaCO20 = 32.6596e-6;     % Solubility of CO2 in water at 37 C, M/mmHg
delTemp = Temp-37;          % Temperature deviation from the standard temperature
alphaO2 = alphaO20*(1 - 1e-2*delTemp + 4.234e-4*delTemp^2);  % Corrected solubility of O2
alphaCO2 = alphaCO20*(1 - 1.86e-2*delTemp + 6.515e-4*delTemp^2); % Corrected solubility of CO2

Hct = 0.45;                 % Volume fraction of RBCs in blood (Hematocrit), unitless
AmHbBl = Hct*150/0.45;      % Amount of hemoglobin in gm per liter of blood, gm/L
MWHb = 64100;               % Molecular weight of hemoglobin, gm/mol
HbBl = AmHbBl/MWHb;         % Concentration of hemoglobin in blood, mol/L or M
HbRbc = HbBl/Hct;           % Concentration of hemoglobin in RBCs, M
DPGrbc = 4.65e-3;           % Concentration of 2,3-DPG in RBCs, M
Hct2 = 0.8;                 % Volume fraction of PCs in systemic tissue, unitless
MbPC = 0.5e-3;              % Concentration of myoglobin in tissue, M
MbTis = Hct2*MbPC;          % Concentration of myoglobin in tissue, M
P50Mb = 2.4;                % P50 of O2 saturation of myoglobin, mmHg
C50Mb = alphaO2*P50Mb;      % C50 of O2 saturation of myoglobin, mmHg

Mol2L = 22.4;           	% Conversion from mole to L of gas; L/mol
Pbaro = 760;                % Barometric pressure of the air, mmHg
PH2O = 47;                  % Partial pressure of H2O in the inspired air, mmHg (T=37oC)
PIO2 = 150;                 % Partial pressure of O2 in the inspired air, mmHg
RQ = 0.8;                   % Respiratory quotient (unitless)
VdotO2 = 0.25;              % O2 uptake rate in the lung; L/min (250 ml/min)
VdotCO2 = RQ*VdotO2;        % CO2 removal rate from the lungs; L/min (200 ml/min)
VdotAlv = 4.2;              % Alveolar ventilation, L/min (with ~12 respiratory cycle/minute)
Qdot = 5.0;                 % Blood flow rate (cardiac output); L/min

VolAlv = 0.35;              % Alveolar volume (tidal volume - dead space), L (VdotAlv/12)   
VolPulCap = 0.07;           % Pulmonary capillary volume, L (~20% of VolAlv)
VolSysCap = 2.1;            % Systemic capillary volume, L (~7% of VolSysTis) 
VolSysTis = 30.0;           % Systemic tissue volume, L
GDrbc = 0.69;               % Gibbs-Donnan ratio across the RBC membrane, unitless
GDcap = 0.63;               % Gibbs-Donnan ratio across the capillary membrane, unitless
GDpc = 0.74;                % Gibbs-Donnan ratio across the PC membrane, unitless

PSO2PulCap = 1320;          % PS for O2 across the pulmonary capillary, L/min
PSO2SysCap = 1000;          % PS for O2 across the systemic capillary, L/min
PSCO2PulCap = 1155;         % PS for CO2 across the pulmonary capillary, L/min
PSCO2SysCap = 875;          % PS for CO2 across the systemic capillary, L/min
PSHCO3mSysCap = 11.55;      % PS for HCO3- across the systemic capillary, L/min
PSHpSysCap = 115.5;         % PS for H+ across the systemic capillary, L/min

CAPulCap = 4000;            % Carbonic anhydrase activity in pulmonary capillary, unitless
CASysCap = 4000;            % Carbonic anhydrase activity in systemic capillary, unitless
CASysTis = 4000;            % Carbonic anhydrase activity in systemic tissue, unitless
BetaPulCap = 24e-3;         % Buffering capacity in pulmonary capillary, M
BetaSysCap = 24e-3;         % Buffering capacity in systemic capillary, M
BetaSysTis = 24e-3;         % Buffering capacity in systemic tissue, M

save CommonParams.mat Temp alphaO2 alphaCO2 Hct HbRbc DPGrbc Hct2 MbTis C50Mb ...
                      Mol2L Pbaro PH2O PIO2 RQ VdotO2 VdotCO2 VdotAlv Qdot VolAlv ...
                      VolPulCap VolSysCap VolSysTis GDrbc GDcap GDpc PSO2PulCap ...0
                      PSO2SysCap PSCO2PulCap PSCO2SysCap PSHCO3mSysCap PSHpSysCap ...
                      CAPulCap CASysCap CASysTis BetaPulCap BetaSysCap BetaSysTis

%% Partial pressures of O2 and CO2 in alevoli under continuous breathing conditions
PO2AlvSS = PIO2 - (VdotO2/VdotAlv)*(Pbaro-PH2O); % Partial pressure of O2 in alveoli
PCO2AlvSS = (VdotCO2/VdotAlv)*(Pbaro-PH2O); % Partial pressure of CO2 in alveoli

%% Compute and prescribe initial conditions for the recirculating gas exchange model
% Partial pressures of gasses in different compartments
PO2Alv0 = 110;              % PO2 in alveoli at t=0; mmHg
PO2PulCap0 = 100;           % PO2 in pulmonary capillary at t=0; mmHg
PO2SysCap0 = 40;            % PO2 in systemic capillary at t=0; mmHg
PO2SysTis0 = 30;            % PO2 in systemic tissue at t=0; mmHg
PCO2Alv0 = 40;              % PCO2 in alveoli at t=0; mmHg
PCO2PulCap0 = 40;           % PCO2 in pulmonary capillary at t=0; mmHg
PCO2SysCap0 = 45;           % PCO2 in systemic capillary at t=0; mmHg
PCO2SysTis0 = 45;           % PCO2 in systemic tissue at t=0; mmHg

% pH and H+ in different compartments
pHPulCapRbc0 = 7.30; pHPulCapPl0 = pHPulCapRbc0-log10(GDrbc);
HpPulCapRbc0 = 10^-pHPulCapRbc0; HpPulCapPl0 = 10^-pHPulCapPl0;
HpPulCap0 = (1-Hct)*HpPulCapPl0+Hct*HpPulCapRbc0;
pHSysCapRbc0 = 7.24; pHSysCapPl0 = pHSysCapRbc0-log10(GDrbc);
HpSysCapRbc0 = 10^-pHSysCapRbc0; HpSysCapPl0 = 10^-pHSysCapPl0;
HpSysCap0 = (1-Hct)*HpSysCapPl0+Hct*HpSysCapRbc0;
HpSysTis0 = HpSysCap0/GDcap;

% HCO3m and total gasses in different compartments
Input = [PO2PulCap0,PCO2PulCap0,pHPulCapRbc0,DPGrbc,Temp,HbRbc,Hct];
[Output] = SHbO2CO2_EJAP2016(Input);
TO2PulCap0 = Output{2}(1); TCO2PulCap0 = Output{2}(2); 
HCO3mPulCap0 = Output{2}(3) - Output{2}(2); 
Input = [PO2SysCap0,PCO2SysCap0,pHSysCapRbc0,DPGrbc,Temp,HbRbc,Hct];
[Output] = SHbO2CO2_EJAP2016(Input);
TO2SysCap0 = Output{2}(1); TCO2SysCap0 = Output{2}(2); 
HCO3mSysCap0 = Output{2}(3) - Output{2}(2);
HCO3mSysTis0 = HCO3mSysCap0*GDcap;

FO2Alv0 = PO2Alv0*alphaO2; FCO2Alv0 = PCO2Alv0*alphaCO2;
FO2SysTis0 = PO2SysTis0*alphaO2; FCO2SysTis0 = PCO2SysTis0*alphaCO2;
TO2SysTis0 = FO2SysTis0 + MbTis*FO2SysTis0/(C50Mb+FO2SysTis0);

%% Solve the mass balance equations (ODEs) governing the recirculating gas exchange
Y0 = [FO2Alv0; TO2PulCap0; TO2SysCap0; TO2SysTis0; FCO2Alv0; TCO2PulCap0; ...
    TCO2SysCap0; FCO2SysTis0; HCO3mPulCap0; HCO3mSysCap0; HCO3mSysTis0; ...
    HpPulCap0; HpSysCap0; HpSysTis0];

options = odeset('RelTol',1e-8, 'AbsTol',1e-10, 'MaxStep',1e-2, 'InitialStep',1e-2,...
    'NonNegative',(1:14), 'MaxOrder',5, 'BDF','on', 'Stats','off');
% options = odeset('RelTol',1e-6,'AbsTol',1e-8);
% options = [];

[T,Y] = ode15s(@Gas_Recirc,[-5:0.1:0],Y0,options); % Run the code to steady stae
Y0 = Y(end,:)'; % Reassign the initial conditions
%[T,Y] = ode15s(@Gas_Recirc,[0:0.1:10],Y0,options);
[T,Y] = ode15s(@Gas_Recirc,[0:0.1:15],Y0,options);

FO2Alv = Y(:,1); TO2PulCap = Y(:,2); TO2SysCap = Y(:,3); TO2SysTis = Y(:,4);
FCO2Alv = Y(:,5); TCO2PulCap = Y(:,6); TCO2SysCap = Y(:,7); FCO2SysTis = Y(:,8);
HCO3mPulCap = Y(:,9); HCO3mSysCap = Y(:,10); HCO3mSysTis = Y(:,11);
HpPulCap = Y(:,12); HpSysCap = Y(:,13); HpSysTis = Y(:,14);

%% Post-process various model simulation results 
Output1 = zeros(length(T),8); Output2 = zeros(length(T),18);
for i = 1:length(T)
    [~,Output1(i,:),Output2(i,:)] = Gas_Recirc(T(i),Y(i,:));
end

PO2Alv = Output1(:,1); PO2PulCap = Output1(:,2); PO2SysCap = Output1(:,3);
PO2SysTis = Output1(:,4); PCO2Alv = Output1(:,5); PCO2PulCap = Output1(:,6); 
PCO2SysCap = Output1(:,7); PCO2SysTis = Output1(:,8);

pHPulCap = Output2(:,1); pHPulCapPl = Output2(:,2); pHPulCapRBC = Output2(:,3); 
pHSysCap = Output2(:,4); pHSysCapPl = Output2(:,5); pHSysCapRBC = Output2(:,6); 
pHSysTis  = Output2(:,7); pHSysTisISF  = Output2(:,8); pHSysTisPC  = Output2(:,9); 
HCO3mPulCap = Output2(:,10); HCO3mPulCapPl = Output2(:,11); HCO3mPulCapRBC = Output2(:,12); 
HCO3mSysCap = Output2(:,13); HCO3mSysCapPl = Output2(:,14); HCO3mSysCapRBC = Output2(:,15); 
HCO3mSysTis = Output2(:,16)'; HCO3mSysTisISF = Output2(:,17)'; HCO3mSysTisPC = Output2(:,18)';  

%% Plot various model simulation results 
figure(1); hold on; box on;
plot(T,PO2Alv,'-m',T,PO2PulCap,'-r',T,PO2SysCap,'-b',T,PO2SysTis,'-g','LineWidth',2)
legend('PO2_{Alveoli}','PO2_{PulCap}','PO2_{SysCap}','PO2_{SysTis}','Location','East')
axis([0 10 20 120])
set(gca,'LineWidth',1.5,'FontName','Arial','FontSize',16);
xlabel('Time, min'); ylabel('PO2, mmHg');

figure(2); hold on; box on;
plot(T,PCO2Alv,'-m',T,PCO2PulCap,'-r',T,PCO2SysCap,'-b',T,PCO2SysTis,'-g','LineWidth',2)
legend('PCO2_{Alveoli}','PCO2_{PulCap}','PCO2_{SysCap}','PCO2_{SysTis}','Location','NorthEast')
axis([0 15 20 60])
set(gca,'LineWidth',1.5,'FontName','Arial','FontSize',16);
xlabel('Time, min'); ylabel('PCO2, mmHg');

figure(3); hold on; box on;
plot(T,pHPulCap,'-r',T,pHSysCap,'-b',T,pHSysTis,'-g','LineWidth',2)
legend('pH_{PulCap}','pH_{SysCap}','pH_{SysTis}','Location','East')
axis([0 15 6.9 7.5])
set(gca,'LineWidth',1.5,'FontName','Arial','FontSize',16);
xlabel('Time, min'); ylabel('pH');

figure(4); hold on; box on;
plot(T,HCO3mPulCap*1e3,'-r',T,HCO3mSysCap*1e3,'-b',T,HCO3mSysTis*1e3,'-g','LineWidth',2)
legend('HCO3m_{PulCap}','HCO3m_{SysCap}','HCO3m_{SysTis}','Location','East')
axis([0 10 8 24])
set(gca,'LineWidth',1.5,'FontName','Arial','FontSize',16);
xlabel('Time, min'); ylabel('HCO3m, mM');
